( cd usr/lib ; rm -rf libmusepack.so )
( cd usr/lib ; ln -sf libmusepack.so.2.0.0 libmusepack.so )
( cd usr/lib ; rm -rf libmusepack.so.2 )
( cd usr/lib ; ln -sf libmusepack.so.2.0.0 libmusepack.so.2 )
